==> manaplus-1.1.7.24.tar.bz2: manaplus/src/net/tmwa/npchandler.cpp <==
/*
 *  The ManaPlus Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *  Copyright (C) 2011  The ManaPlus Developers
 *
 *  This file is part of The ManaPlus Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "net/tmwa/npchandler.h"

#include "actorspritemanager.h"
#include "localplayer.h"

#include "gui/npcdialog.h"

#include "net/messagein.h"
#include "net/messageout.h"
#include "net/net.h"
#include "net/npchandler.h"

#include "net/tmwa/protocol.h"

#include <SDL_types.h>

#include "debug.h"

extern Net::NpcHandler *npcHandler;

namespace TmwAthena
{

NpcHandler::NpcHandler()
{
    static const Uint16 _messages[] =
    {
        SMSG_NPC_CHOICE,
        SMSG_NPC_MESSAGE,
        SMSG_NPC_NEXT,
        SMSG_NPC_CLOSE,
        SMSG_NPC_INT_INPUT,
        SMSG_NPC_STR_INPUT,
        0
    };
    handledMessages = _messages;
    npcHandler = this;
}

void NpcHandler::handleMessage(Net::MessageIn &msg)
{
    if (msg.getId() == SMSG_NPC_CHOICE || msg.getId() == SMSG_NPC_MESSAGE)
        msg.readInt16();  // length

    int npcId = msg.readInt32();
    NpcDialogs::iterator diag = mNpcDialogs.find(npcId);
    NpcDialog *dialog = 0;

    if (diag == mNpcDialogs.end())
    {
        // Empty dialogs don't help
        if (msg.getId() == SMSG_NPC_CLOSE)
        {
            closeDialog(npcId);
            return;
        }
        else if (msg.getId() == SMSG_NPC_NEXT)
        {
            nextDialog(npcId);
            return;
        }
        else
        {
            dialog = new NpcDialog(npcId);
            Wrapper wrap;
            wrap.dialog = dialog;
            mNpcDialogs[npcId] = wrap;
        }
    }
    else
    {
        dialog = diag->second.dialog;
    }

    switch (msg.getId())
    {
        case SMSG_NPC_CHOICE:
            if (dialog)
            {
                dialog->choiceRequest();
                dialog->parseListItems(msg.readString(msg.getLength() - 8));
            }
            else
            {
                msg.readString(msg.getLength() - 8);
            }
            break;

        case SMSG_NPC_MESSAGE:
            if (dialog)
                dialog->addText(msg.readString(msg.getLength() - 8));
            else
                msg.readString(msg.getLength() - 8);
            break;

        case SMSG_NPC_CLOSE:
            // Show the close button
            if (dialog)
                dialog->showCloseButton();
            break;

        case SMSG_NPC_NEXT:
            // Show the next button
            if (dialog)
                dialog->showNextButton();
            break;

        case SMSG_NPC_INT_INPUT:
            // Request for an integer
            if (dialog)
                dialog->integerRequest(0);
            break;

        case SMSG_NPC_STR_INPUT:
            // Request for a string
            if (dialog)
                dialog->textRequest("");
            break;

        default:
            break;
    }

    if (player_node && player_node->getCurrentAction() != Being::SIT)
        player_node->setAction(Being::STAND);
}

void NpcHandler::talk(int npcId)
{
    MessageOut outMsg(CMSG_NPC_TALK);
    outMsg.writeInt32(npcId);
    outMsg.writeInt8(0); // Unused
}

void NpcHandler::nextDialog(int npcId)
{
    MessageOut outMsg(CMSG_NPC_NEXT_REQUEST);
    outMsg.writeInt32(npcId);
}

void NpcHandler::closeDialog(int npcId)
{
    MessageOut outMsg(CMSG_NPC_CLOSE);
    outMsg.writeInt32(npcId);

    NpcDialogs::iterator it = mNpcDialogs.find(npcId);
    if (it != mNpcDialogs.end())
    {
        if ((*it).second.dialog)
            (*it).second.dialog->close();
        mNpcDialogs.erase(it);
    }
}

void NpcHandler::listInput(int npcId, unsigned char value)
{
    MessageOut outMsg(CMSG_NPC_LIST_CHOICE);
    outMsg.writeInt32(npcId);
    outMsg.writeInt8(static_cast<unsigned char>(value));
}

void NpcHandler::integerInput(int npcId, int value)
{
    MessageOut outMsg(CMSG_NPC_INT_RESPONSE);
    outMsg.writeInt32(npcId);
    outMsg.writeInt32(value);
}

void NpcHandler::stringInput(int npcId, const std::string &value)
{
    MessageOut outMsg(CMSG_NPC_STR_RESPONSE);
    outMsg.writeInt16(static_cast<Sint16>(value.length() + 9));
    outMsg.writeInt32(npcId);
    outMsg.writeString(value, static_cast<int>(value.length()));
    outMsg.writeInt8(0); // Prevent problems with string reading
}

void NpcHandler::sendLetter(int npcId A_UNUSED,
                            const std::string &recipient A_UNUSED,
                            const std::string &text A_UNUSED)
{
    // TODO
}

void NpcHandler::startShopping(int beingId A_UNUSED)
{
    // TODO
}

void NpcHandler::buy(int beingId)
{
    MessageOut outMsg(CMSG_NPC_BUY_SELL_REQUEST);
    outMsg.writeInt32(beingId);
    outMsg.writeInt8(0); // Buy
}

void NpcHandler::sell(int beingId)
{
    MessageOut outMsg(CMSG_NPC_BUY_SELL_REQUEST);
    outMsg.writeInt32(beingId);
    outMsg.writeInt8(1); // Sell
}

void NpcHandler::buyItem(int beingId A_UNUSED, int itemId,
                         unsigned char color, int amount)
{
    MessageOut outMsg(CMSG_NPC_BUY_REQUEST);
    if (serverVersion > 0)
    {
        outMsg.writeInt16(10); // One item (length of packet)
        outMsg.writeInt16(static_cast<Sint16>(amount));
        outMsg.writeInt16(static_cast<Sint16>(itemId));
        outMsg.writeInt8(color);
        outMsg.writeInt8(0);
    }
    else
    {
        outMsg.writeInt16(8); // One item (length of packet)
        outMsg.writeInt16(static_cast<Sint16>(amount));
        outMsg.writeInt16(static_cast<Sint16>(itemId));
    }
}

void NpcHandler::sellItem(int beingId A_UNUSED, int itemId, int amount)
{
    MessageOut outMsg(CMSG_NPC_SELL_REQUEST);
    outMsg.writeInt16(8); // One item (length of packet)
    outMsg.writeInt16(static_cast<Sint16>(itemId + INVENTORY_OFFSET));
    outMsg.writeInt16(static_cast<Sint16>(amount));
}

void NpcHandler::endShopping(int beingId A_UNUSED)
{
    // TODO
}

void NpcHandler::clearDialogs()
{
    NpcDialogs::iterator it = mNpcDialogs.begin();
    NpcDialogs::iterator it_end = mNpcDialogs.end();
    while (it != it_end)
    {
        delete (*it).second.dialog;
        ++ it;
    }
    mNpcDialogs.clear();
}

} // namespace TmwAthena

==> manaplus-1.1.8.7.tar.bz2: manaplus/src/net/tmwa/npchandler.cpp <==
/*
 *  The ManaPlus Client
 *  Copyright (C) 2004-2009  The Mana World Development Team
 *  Copyright (C) 2009-2010  The Mana Developers
 *  Copyright (C) 2011  The ManaPlus Developers
 *
 *  This file is part of The ManaPlus Client.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "net/tmwa/npchandler.h"

#include "localplayer.h"

#include "gui/npcdialog.h"

#include "net/messagein.h"
#include "net/net.h"
#include "net/npchandler.h"

#include "net/tmwa/protocol.h"

#include "net/ea/eaprotocol.h"

#include "debug.h"

extern Net::NpcHandler *npcHandler;

namespace TmwAthena
{

NpcHandler::NpcHandler()
{
    static const Uint16 _messages[] =
    {
        SMSG_NPC_CHOICE,
        SMSG_NPC_MESSAGE,
        SMSG_NPC_NEXT,
        SMSG_NPC_CLOSE,
        SMSG_NPC_INT_INPUT,
        SMSG_NPC_STR_INPUT,
        0
    };
    handledMessages = _messages;
    npcHandler = this;
}

void NpcHandler::handleMessage(Net::MessageIn &msg)
{
    getNpc(msg, msg.getId() == SMSG_NPC_CHOICE
        || msg.getId() == SMSG_NPC_MESSAGE);

    switch (msg.getId())
    {
        case SMSG_NPC_CHOICE:
            processNpcChoice(msg);
            break;

        case SMSG_NPC_MESSAGE:
            processNpcMessage(msg);
            break;

        case SMSG_NPC_CLOSE:
            processNpcClose(msg);
            break;

        case SMSG_NPC_NEXT:
            processNpcNext(msg);
            break;

        case SMSG_NPC_INT_INPUT:
            processNpcIntInput(msg);
            break;

        case SMSG_NPC_STR_INPUT:
            processNpcStrInput(msg);
            break;

        default:
            break;
    }

    if (player_node && player_node->getCurrentAction() != Being::SIT)
        player_node->setAction(Being::STAND);
    
    mDialog = 0;
}

void NpcHandler::talk(int npcId)
{
    MessageOut outMsg(CMSG_NPC_TALK);
    outMsg.writeInt32(npcId);
    outMsg.writeInt8(0); // Unused
}

void NpcHandler::nextDialog(int npcId)
{
    MessageOut outMsg(CMSG_NPC_NEXT_REQUEST);
    outMsg.writeInt32(npcId);
}

void NpcHandler::closeDialog(int npcId)
{
    MessageOut outMsg(CMSG_NPC_CLOSE);
    outMsg.writeInt32(npcId);

    NpcDialogs::iterator it = mNpcDialogs.find(npcId);
    if (it != mNpcDialogs.end())
    {
        if ((*it).second.dialog)
            (*it).second.dialog->close();
        mNpcDialogs.erase(it);
    }
}

void NpcHandler::listInput(int npcId, unsigned char value)
{
    MessageOut outMsg(CMSG_NPC_LIST_CHOICE);
    outMsg.writeInt32(npcId);
    outMsg.writeInt8(static_cast<unsigned char>(value));
}

void NpcHandler::integerInput(int npcId, int value)
{
    MessageOut outMsg(CMSG_NPC_INT_RESPONSE);
    outMsg.writeInt32(npcId);
    outMsg.writeInt32(value);
}

void NpcHandler::stringInput(int npcId, const std::string &value)
{
    MessageOut outMsg(CMSG_NPC_STR_RESPONSE);
    outMsg.writeInt16(static_cast<Sint16>(value.length() + 9));
    outMsg.writeInt32(npcId);
    outMsg.writeString(value, static_cast<int>(value.length()));
    outMsg.writeInt8(0); // Prevent problems with string reading
}

void NpcHandler::buy(int beingId)
{
    MessageOut outMsg(CMSG_NPC_BUY_SELL_REQUEST);
    outMsg.writeInt32(beingId);
    outMsg.writeInt8(0); // Buy
}

void NpcHandler::sell(int beingId)
{
    MessageOut outMsg(CMSG_NPC_BUY_SELL_REQUEST);
    outMsg.writeInt32(beingId);
    outMsg.writeInt8(1); // Sell
}

void NpcHandler::buyItem(int beingId A_UNUSED, int itemId,
                         unsigned char color, int amount)
{
    MessageOut outMsg(CMSG_NPC_BUY_REQUEST);
    if (serverVersion > 0)
    {
        outMsg.writeInt16(10); // One item (length of packet)
        outMsg.writeInt16(static_cast<Sint16>(amount));
        outMsg.writeInt16(static_cast<Sint16>(itemId));
        outMsg.writeInt8(color);
        outMsg.writeInt8(0);
    }
    else
    {
        outMsg.writeInt16(8); // One item (length of packet)
        outMsg.writeInt16(static_cast<Sint16>(amount));
        outMsg.writeInt16(static_cast<Sint16>(itemId));
    }
}

void NpcHandler::sellItem(int beingId A_UNUSED, int itemId, int amount)
{
    MessageOut outMsg(CMSG_NPC_SELL_REQUEST);
    outMsg.writeInt16(8); // One item (length of packet)
    outMsg.writeInt16(static_cast<Sint16>(itemId + INVENTORY_OFFSET));
    outMsg.writeInt16(static_cast<Sint16>(amount));
}

int NpcHandler::getNpc(Net::MessageIn &msg, bool haveLength)
{
    if (haveLength)
        msg.readInt16();  // length

    const int npcId = msg.readInt32();

    NpcDialogs::iterator diag = mNpcDialogs.find(npcId);
    mDialog = 0;

    if (diag == mNpcDialogs.end())
    {
        // Empty dialogs don't help
        if (msg.getId() == SMSG_NPC_CLOSE)
        {
            closeDialog(npcId);
            return npcId;
        }
        else if (msg.getId() == SMSG_NPC_NEXT)
        {
            nextDialog(npcId);
            return npcId;
        }
        else
        {
            mDialog = new NpcDialog(npcId);
            Wrapper wrap;
            wrap.dialog = mDialog;
            mNpcDialogs[npcId] = wrap;
        }
    }
    else
    {
        mDialog = diag->second.dialog;
    }
    return npcId;
}

} // namespace TmwAthena
